<?php
/* vim: set expandtab tabstop=4 softtabstop=4 shiftwidth=4:
  Codificación: UTF-8
  +----------------------------------------------------------------------+
  | Elastix version 2.4.0-1                                              |
  | http://www.elastix.org                                               |
  +----------------------------------------------------------------------+
  | Copyright (c) 2006 Palosanto Solutions S. A.                         |
  +----------------------------------------------------------------------+
  | Cdla. Nueva Kennedy Calle E 222 y 9na. Este                          |
  | Telfs. 2283-268, 2294-440, 2284-356                                  |
  | Guayaquil - Ecuador                                                  |
  | http://www.palosanto.com                                             |
  +----------------------------------------------------------------------+
  | The contents of this file are subject to the General Public License  |
  | (GPL) Version 2 (the "License"); you may not use this file except in |
  | compliance with the License. You may obtain a copy of the License at |
  | http://www.opensource.org/licenses/gpl-license.php                   |
  |                                                                      |
  | Software distributed under the License is distributed on an "AS IS"  |
  | basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See  |
  | the License for the specific language governing rights and           |
  | limitations under the License.                                       |
  +----------------------------------------------------------------------+
  | The Original Code is: Elastix Open Source.                           |
  | The Initial Developer of the Original Code is PaloSanto Solutions    |
  +----------------------------------------------------------------------+
  $Id: paloSantoOpenVPN.class.php,v 1.1 2013-06-10 07:10:30 Nikita Rukavkov admin@voip-lab.ru Exp $ */

include_once("libs/paloSantoDB.class.php");
include_once("libs/paloSantoConfig.class.php");
include_once("Telnet.class.php");

/* Class OpenVPN Server */
class PaloSantoOpenVPN
{
    var $_DB;
    var $errMsg;
    var $pConfig;
    var $oConfig;
    var $rsa_path;
    var $mConfig;

    function PaloSantoOpenVPN(&$pDB)
    {
	if (is_object($pDB)) {
	    $this->_DB =& $pDB;
	    $this->errMsg = $this->_DB->errMsg;
	} else {
	    $dsn = (string)$pDB;
	    $this->_DB = new paloDB($dsn);
	    if (!$this->_DB->connStatus) {
		$this->errMsg = $this->_DB->errMsg;
	    } 
	}
	$this->mConfig = array();
	$this->mConfig['pidpath'] = "/var/run/openvpn_client";
    }
    

    function connection_exists($configname)
    {
	$result = $this->_DB->fetchTable("SELECT id FROM clients WHERE name = '$configname'", true);
	return ( count ($result) > 0 ? TRUE : FALSE);
    }

    //////////////////ADD CONFIGURATION FOR OPENVPN CLIENT CONFIG/////////////////////////
    function addConfigOpenVPN($configname, $tmppath, $autostart)
    {
	if(!file_exists($tmppath)) return FALSE;

        $sComando = '/usr/bin/elastix-helper openvpn_client --addconfig'.
                    ' --tmppath '.escapeshellarg($tmppath).
                    ' --filename '.escapeshellarg($configname).
                    ' 2>&1';
        $output = $ret = NULL;
        
	exec($sComando, $output, $ret);
        if ($ret != 0) {
                $this->errMsg = implode('', $output);
		return FALSE;
        }
        else
        {
    	    $query = "INSERT INTO clients (name,State) VALUES (?,?)";
	    $arrParam = array($configname,$autostart);
	    $result = $this->_DB->genQuery($query,$arrParam);
	    
	    //Adding management option to config file
	    $result = $this->_DB->fetchTable("SELECT id FROM clients WHERE name = '$configname'", true);
	    $id = $result[0][id];
	    
    	    $sComando = '/usr/bin/elastix-helper openvpn_client --modconfig'.
                    ' --filename '.escapeshellarg($configname).
                    ' --id '.escapeshellarg($id).
                    ' 2>&1';
	        
            $output = $ret = NULL;
        
	    exec($sComando, $output, $ret);
	    
            return TRUE;
        }
    }
    //////////////////////////////////////////////////////////////////////////////////////


    function load_client_table()
    {
	global $arrLang;
	$rTable = array();
	
	$query   = "SELECT * FROM clients";
	$result = $this->_DB->fetchTable($query, true);
	if($result == FALSE)
	{
	    $this->errMsg = $this->_DB->errMsg;
	    return array();
	}
	
	foreach ($result as $res)
	{   
	    
	    $conn = NULL;
	    $readbytes = $writebytes = 0;
	    $state = array('-','-','-','-','-','-');
	    $autorun = "<a href=index.php?menu=openvpn_client&clientid=$res[id]&reconnect=1>".($res['State'] == 'TRUE' ? $arrLang['ON'] : $arrLang['OFF'])."</a>";
	    
	    if(!file_exists($this->mConfig['pidpath']."/".$res['name'].".pid"))
		$connectLink = '<a href=index.php?menu=openvpn_client&connect=1&name='.$res[name].'>'._('Connect').'</a>';
	    else
	    {
		try{
		    //CHECK CONNECTION STATE
		    $state = NULL;
		    $conn = new Telnet("127.0.0.1",$res['id']+7000,1,"");
		    $conn->clearBuffer();
		
		    $state_str=$conn->exec("state");
		    foreach(explode("\n", $state_str) as $line) {
			$state_tmp = explode(",", $line);
			if(is_numeric($state_tmp[0]))
			{
			    $state = $state_tmp;
			    break;
			}
		    }
		    
		    //CHECK DOWNLOAD/UPLOAD TRAFFIC
		    $conn->clearBuffer();
		    $state_str=$conn->exec("status 2");
		    
		    foreach(explode("\n", $state_str) as $line) {
			$state_tmp = explode(",", $line);
			switch ($state_tmp[0])
			{
			    case 'TCP/UDP read bytes':
				$readbytes+=$state_tmp[1];
				break;
			    case 'TCP/UDP write bytes':
				$writebytes+=$state_tmp[1];
				break;
			}    
		    }
		}
	        catch(Exception $e)
		{
		}
		
		$conn = NULL;
		$connectLink = '<a href=index.php?menu=openvpn_client&connect=0&name='.$res[name].'>'._('Disconnect').'</a>';
	    }
	    
	    //MAIL CONFIG 
	    $output = $ret = NULL;

	    //SERVER IP/DNS CONFIG
            $sComando = '/usr/bin/elastix-helper openvpn_client --dumpserver --config '.escapeshellarg($res['name']).' 2>&1';
	    exec($sComando, $soutput, $ret);
	    
	    $rTable[] = array("<input type=\"checkbox\" name=\"config[]\" value=\"$res[name]\" />",$res[id],$res[name],$autorun,$soutput[0],$state[3],$state[4],round($readbytes/(1024*1024),2),round($writebytes/(1024*1024),2),(($state[0]!='-')? date('D F d H:i:s Y',$state[0]) : "-"),$state[1],$connectLink);
	}

	return $rTable;
    }

    function connectOpenVPNClient($connection_name)
    {
        $sComando = '/usr/bin/elastix-helper openvpn_client --connect'.
                    ' --config '.escapeshellarg($connection_name).
                    ' 2>&1';
        $output = $ret = NULL;
        
	exec($sComando, $output, $ret);
        if ($ret != 0) {
                $this->errMsg = implode('', $output);
		return FALSE;
        }
        else
        {
            return TRUE;
        }
	
    }

    function reconnectOpenVPNClient($id)
    {
	$result = $this->_DB->fetchTable("SELECT State FROM clients WHERE id = $id", true);
	$val = ($result[0][State] == 'TRUE' ? 'FALSE' : 'TRUE');
	$query = "UPDATE clients SET State='$val' WHERE id=$id";
	$result = $this->_DB->genQuery($query);
    }

    
    function disconnectOpenVPNClient($connection_name)
    {
        $sComando = '/usr/bin/elastix-helper openvpn_client --disconnect'.
                    ' --config '.escapeshellarg($connection_name).
                    ' 2>&1';
        $output = $ret = NULL;
        
	exec($sComando, $output, $ret);
	return TRUE;
    }

   function downloadLogs()
   {
        $output = $ret = NULL;
    	$sComando = '/usr/bin/elastix-helper openvpn_client --ziplogs 2>&1';
    	                            
        exec($sComando, $output, $ret);
        if($ret != 0) return FALSE;
        else return TRUE;
    }

    function getConfigurationOpenVPN() 
    {
        global $arrLang;
    }

    function createLogsArchiveOpenVPN()
    {
        $this->errMsg = '';
        $ret = NULL;
        $ret = split("\n", `/usr/bin/elastix-helper openvpn_client --dumplogs`);
        return $ret;
    }

    function removeFromOpenVPN($configs)
    {
	foreach($configs as $config)
	{
	    $this->disconnectOpenVPNClient($config);
	    $query = "DELETE FROM clients WHERE name=?";
	    $arrParam = array($config);
	    $result = $this->_DB->genQuery($query,$arrParam);
	    `/usr/bin/elastix-helper openvpn_client --removeconfig --config $config`;
	    
	}
    }

    function checkInstall($namePackage)
    {
    	global $arrLang;
    	$this->errMsg = '';
	exec("rpm -q $namePackage",$respuesta,$retorno);
	if($retorno == 0){
	    return TRUE;
	}
	else{
	    return FALSE;
	}
    }
    
}
?>
